#!/usr/bin/python

import sys, os, MySQLdb, re, ConfigParser
from gui import MainForm, Qwt
from qt import *
from Qwt5.anynumpy import *

class MyForm(MainForm):

    """
        Main class, inhereting the GUI (MainForm) and adding update control
        through QTimer.
    """

    def __init__(self, *args, **kw):

        """
            Upon initialization, start Timer, define datahandler and show table
            view per default
        """

        MainForm.__init__(self, *args, **kw)

        
        self.setCaption(config.get("gui","system_name"))
        

        # Initialize the Temperature Plot
        p1 = self.TemperaturePlot
        self.alignScales(p1)

        p1.t = arange(-3600.1, 0.0, 0.5)
        p1.x = ones(len(p1.t), Float)*-120
        p1.y = ones(len(p1.t), Float)*-180

        p1.ccdtemp = Qwt.QwtPlotCurve("CCD Temp")
        p1.ccdtemp.attach(p1)
        p1.ccdtemp.setPen(QPen(Qt.red))

        p1.ln2temp = Qwt.QwtPlotCurve("LN2 Temp")
        p1.ln2temp.attach(p1)
        p1.ln2temp.setPen(QPen(Qt.blue))

        p1.setAxisScale(Qwt.QwtPlot.xBottom, -3600, 0, 600)
        #p1.setAxisScale(Qwt.QwtPlot.yLeft, -1, 1, 0.5)

        p1.setAxisTitle(Qwt.QwtPlot.xBottom, " ")
        p1.setAxisTitle(Qwt.QwtPlot.yLeft, "<font size=-2><font color='red'>CCD</font>,<font color='blue'>Dewar</font> Temp. (C)</font>")



        # Initialize the Pressure Plot
        p2 = self.PressurePlot

        logplot = Qwt.QwtLog10ScaleEngine()
        p2.setAxisScaleEngine(Qwt.QwtPlot.yLeft,logplot)

        self.alignScales(p2)

        p2.t = arange(-3600.1, 0.0, 0.5)
        p2.y = ones(len(p2.t), Float)*10e-5

        p2.plot = Qwt.QwtPlotCurve(" ")
        p2.plot.attach(p2)
        p2.plot.setPen(QPen(Qt.blue))

        p2.setAxisScale(Qwt.QwtPlot.xBottom, -3600, 0, 600)
        p2.setAxisScale(Qwt.QwtPlot.yLeft, 10e-7, 10e-4)


        p2.setAxisTitle(Qwt.QwtPlot.xBottom, " ")
        p2.setAxisTitle(Qwt.QwtPlot.yLeft, "<font size=-2><font color='blue'>Pressure</font> (mBar)")



        # Setup the Timer
        try:
            self.myTimer = QTimer(self)
            self.connect(self.myTimer, SIGNAL("timeout()"),self.updateStatus)
            self.myTimer.start(int(config.get("gui","refresh_interval")))
        except KeyboardInterrupt:
            self.close()
            sys.exit(1)

	try:
          self.dh = MySQLhandler()
        except DBException, e:
	  print e 
	  sys.exit (1)


    def alignScales(self,plot):
        plot.canvas().setFrameStyle(QFrame.Box | QFrame.Plain)
        plot.canvas().setLineWidth(1)
        for i in range(Qwt.QwtPlot.axisCnt):
            scaleWidget = plot.axisWidget(i)
            if scaleWidget:
                scaleWidget.setMargin(0)
            scaleDraw = plot.axisScaleDraw(i)
            if scaleDraw:
                scaleDraw.enableComponent(
                    Qwt.QwtAbstractScaleDraw.Backbone, False)


    def close(self, *args, **kw):

        """
            Redefine the close() method so we can close datahandler
            connections before closing the form. This works both for
            exit via menu and exit via window manager
        """
        
        self.dh.close()
        return MainForm.close(self, *args, **kw)
    

    def updateStatus(self):
    
        """
            Routine called by QTimer, responsible for retrieving
            up-to-date status values and updating the GUI
        """
        
	try:
	    status     = self.dh.queryCCDStatus()

            r,g,b = 0,170,127
	    if status['DATE_AGE'] > int(config.get("gui","data_age_limit")): 
		r,g,b = 255,0,0
	    self.DATA_AGE.setPaletteForegroundColor(QColor(r,g,b))
	    self.DATA_AGE.setText(str(status['DATE_AGE'])+'s')
	except DBException, e:
	    self.DATA_AGE.setPaletteForegroundColor(QColor(255,0,0))
	    self.DATA_AGE.setText('ERR')
	    return
        except KeyboardInterrupt:
            self.close()
            sys.exit(1)
        
	dict_camera = { 0 : 'Idle',
                        1 : 'Integrating',
                        2 : 'Readout',
                        3 : 'Clear',
                        4 : 'Shutter delay'
                      }

	dict_shutter = { 0 : 'Off',
                         1 : 'On'
                       }


        dict_ampl = { 1 : 'A',
                      2 : 'B',
		      3 : 'AB'
                    }  

        dict_autosave = { 0 : 'OFF',
                          1 : 'ON'
                        }  
    
    
    #t = self.tab_status.page(0)
    #tab_status.removeTab(1)

        if status['HOLD'] == 1:
	    self.CAMERA_MODE.setText("Hold")
	    self.CAMERA_MODE.setPaletteForegroundColor(QColor("red"))
        else:
	    self.CAMERA_MODE.setText(dict_camera.get(status['CAMERA_MODE'],'Undefined'))
    	    self.CAMERA_MODE.setPaletteForegroundColor(QColor(0,170,127))
	
        self.SHUTTER.setText(dict_shutter.get(status['SHUTTER'],'Undefined'))   
        self.AMPL_MODE0.setText(dict_ampl.get(status['AMPL_MODE0'],'Undefined'))   
        self.TINT_RES.setText("%.0f s" % (status['TINT_RES']/1000.0))
        self.TINT.setText("%.0f s" % (status['TINT']/1000.0))
        self.MULTIEXP.setText("%s / %s" % (status['MULTACT'],status['MULTITOT']))
        self.EXP_PROGRESS.setProgress(status['EXP_PROGRESS'])
	self.EXP_PROGRESS_2.setProgress(status['EXP_PROGRESS'])
	self.READ_PROGRESS.setProgress(status['READ_PROGRESS'])
	self.READ_PROGRESS_2.setProgress(status['READ_PROGRESS'])
	self.MEM_PROGRESS_2.setProgress(status['MEM_PROGRESS'])
	self.DESC_PROGRESS.setProgress(status['DESC_PROGRESS'])
	self.FILE_PROGRESS_2.setProgress(status['FILE_PROGRESS'])
	self.SPEED_LABEL.setText(str(status['READ_SPEED']))
        self.XSIZE.setText("XSIZ<BR>" + str(status['XSIZE']))
        self.YSIZE.setText("YSIZ " + str(status['YSIZE']))
        self.XBEGIN.setText( "XBEG<BR>" + str(status['XBEGIN']))
        self.YBEGIN.setText("YBEG " + str(status['YBEGIN']))
	self.XTOT.setText("XTOT " + str(status['XTOT']))
	self.YTOT.setText("YTOT<BR>" + str(status['YTOT']))
	self.XPHY.setText("XPHY " + str(status['XPHY']))
	self.YPHY.setText("YPHY<BR>" + str(status['YPHY']))
	self.BINNING.setText(str(status['XBIN']) + ":" + str(status['YBIN']))
	self.OVERSCAN.setText(str(status['XOVER']) + ":" + str(status['YOVER']))
	if status['AMPL_MODE0'] == 1:
	    self.label_amp_a.setText("A")
	    self.label_amp_b.setText( "")
	elif status['AMPL_MODE0'] == 2:
	    self.label_amp_a.setText("")
	    self.label_amp_b.setText("B")
	elif status['AMPL_MODE0'] == 3:
	    self.label_amp_a.setText("A")
	    self.label_amp_b.setText("B")
	else:
	    self.label_amp_a.setText("err")
	    self.label_amp_b.setText("err")
        self.CCDTEMP.setText(str(status['CCDTEMP']))
        self.REFTEMP.setText(str(status['REFTEMP']))
        self.DEWARTEMP.setText(str(status['DEWARTEMP']))
        self.DEWARPRESS.setText(str(status['DEWARPRESS']))
        self.AUTOSAVE.setText(dict_autosave.get(status['AUTOSAVE'],'Undefined'))
	self.CUR_FILENAME.setText(str(status['IMPATH']) + str(status['CUR_FILENAME']))
	self.TSAM.setText(str(status['TSAM']))
	self.TSPW.setText(str(status['TSPW']))
	self.TSOL.setText(str(status['TSOL']))
	self.TSTR.setText(str(status['TSTR']))
	self.TSND.setText(str(status['TSND']))
	self.TPPW.setText(str(status['TPPW']))
	self.TPOL.setText(str(status['TPOL']))
	self.TPTR.setText(str(status['TPTR']))
	self.CDSG.setText(str(status['CDSG']))
	self.VBHA0.setText(str(status['VBHA0']))
	self.VBHB0.setText(str(status['VBHB0']))
	self.VBHC0.setText(str(status['VBHC0']))
	self.VBLA0.setText(str(status['VBLA0']))
	self.VBLB0.setText(str(status['VBLB0']))
	self.GAIN0.setText(str(status['GAIN0']))
	self.OFFS0.setText(str(status['OFFS0']))
	self.ZERO0.setText(str(status['ZERO0']))
	self.VBHA1.setText(str(status['VBHA1']))
	self.VBHB1.setText(str(status['VBHB1']))
	self.VBHC1.setText(str(status['VBHC1']))
	self.VBLA1.setText(str(status['VBLA1']))
	self.VBLB1.setText(str(status['VBLB1']))
	self.GAIN1.setText(str(status['GAIN1']))
	self.OFFS1.setText(str(status['OFFS1']))
	self.ZERO1.setText(str(status['ZERO1']))
	self.VBHA2.setText(str(status['VBHA2']))
	self.VBHB2.setText(str(status['VBHB2']))
	self.VBHC2.setText(str(status['VBHC2']))
	self.VBLA2.setText(str(status['VBLA2']))
	self.VBLB2.setText(str(status['VBLB2']))
	self.GAIN2.setText(str(status['GAIN2']))
	self.OFFS2.setText(str(status['OFFS2']))
	self.ZERO2.setText(str(status['ZERO2']))
	self.VBHA3.setText(str(status['VBHA3']))
	self.VBHB3.setText(str(status['VBHB3']))
	self.VBHC3.setText(str(status['VBHC3']))
	self.VBLA3.setText(str(status['VBLA3']))
	self.VBLB3.setText(str(status['VBLB3']))
	self.GAIN3.setText(str(status['GAIN3']))
	self.OFFS3.setText(str(status['OFFS3']))
	self.ZERO3.setText(str(status['ZERO3']))



        #
        # Update the Temperature Plot
        #

        p1 = self.TemperaturePlot

        # Shift x array left and assign new value to x[n-1].
        p1.x = concatenate((p1.x[1:], p1.x[:1]), 1)
        p1.x[-1] = status['CCDTEMP'] 
        #p1.x[-1] = -120.0 + 2.0*random.random()
 
        # Shift y array left and assign new value to y[n-1].
        p1.y = concatenate((p1.y[1:], p1.y[:1]), 1)
        p1.y[-1] = status['DEWARTEMP']
        #p1.y[-1] = -188.0 + 2.0*random.random()

        p1.ccdtemp.setData(p1.t, p1.x)
        p1.ln2temp.setData(p1.t, p1.y)

        p1.replot()


        #
        # Update the Pressure Plot
        #

        p2 = self.PressurePlot
    
        # Shift y array left and assign new value to y[n-1].
        p2.y = concatenate((p2.y[1:], p2.y[:1]), 1)
        #p2.y[-1] = status['DEWARPRESS']
        p2.y[-1] = 10e-5 + 10e-6*random.random()

        p2.plot.setData(p2.t, p2.y)

        p2.replot()



class DBException(Exception):
  pass


class MySQLhandler:

    """ 
        Wrapper class for retrieving CCD status values from a MySQL databases 
    """

    def __init__(self):

        """ 
            Initialize the MySQL connection
        """



	try:
	  # Create a persistent database connection 
          self.__cnx = MySQLdb.connect(host=config.get("mysql","host"),
                                       user=config.get("mysql","user"),
                                       passwd=config.get("mysql","passwd"),
                                       db=config.get("mysql","db")
                                       )

 	except MySQLdb.Error, e: 
          raise DBException ("Fatal Error: %s (%d)" % (e.args[1], e.args[0]))


    def tuble2dict(self,cursor):

        """
            Convert a tuble of dictionaries into a dictionary
        """

        a = cursor.fetchall()
        dict = {}
        for i in range(len(a)):
            key = a[i]['pos']
            val = a[i]['name']
            dict[key] = val
        return dict


    def close(self):

        """
            Close database connection
        """    

        if self.__cnx is not None:
            self.__cnx.close()
            self.__cnx = None


    def queryCCDStatus(self):

        """
            Return a dictionary of CCD status values to be shown in the GUI
        """

	try:
	    status = {}
	    cursor = self.__cnx.cursor(MySQLdb.cursors.DictCursor)
	    cursor.execute('SELECT *, TIME_TO_SEC(TIMEDIFF(NOW(),TS)) AS DATE_AGE FROM ccd3_status')
	    self.__cnx.commit()
	    a = cursor.fetchone()

	    return a
    
	except Exception, e: 
          raise DBException ("Error: %s (%s)" % (str(e.__class__), str(e)))
        
          


class SQLitehandler:

    """ 
        Wrapper class for retrieving CCD status values from a SQLite databases 
    """

    pass




if __name__ == "__main__":

    config = ConfigParser.ConfigParser()

    if os.path.exists("ccdstatus.cfg"):
       config.read("ccdstatus.cfg")
       print "Using configuration file in current directory"
    elif os.path.exists("/etc/ccd3/ccdstatus.cfg"):
       config.read("/etc/ccd3/ccdstatus.cfg")
       print "Using configuration file in /etc/ccd3/"
    else:   
       print "Error: Can not find configuration file"
       sys.exit(1)

    try:
        a = QApplication(sys.argv)
        QObject.connect(a,SIGNAL("lastWindowClosed()"),a,SLOT("quit()"))
        w = MyForm()
        a.setMainWidget(w)
        w.show()
        a.exec_loop()
    except KeyboardInterrupt:    
        w.close()
        sys.exit(1)
